{ Tool for timing evaluation of Delphi Pascal code fragments.
  Timing is based on the cpu clock , resolution  ca 10ns (=10^-8 s).
  July 3st 1997 , version 1.10 published as freeware by author A. Baars
  , any comments :  d950021@icpc00.icpc.fukui-u.ac.jp              }

unit Z_prof;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, Buttons, ExtCtrls, StdCtrls;

const nmarks=20;
type
markrecord=record
  passes,thigh,tlow:dword;
  totaltime,fastest:extended;
  active:boolean;
  end;

Tprofiler = class(TForm)
    Panel1: TPanel;
    Clearbutton: TSpeedButton;
    DrawGrid1: TDrawGrid;
    RadioGroup1: TRadioGroup;
    Label1: TLabel;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure ClearbuttonClick(Sender: TObject);
    procedure RadioGroup1Click(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure SpeedButton2Click(Sender: TObject);
  private
  rmark:array[1..nmarks] of markrecord;
  procedure gettime(nr:integer);
  function gettimediff(nr:integer):extended;
  procedure CorrectForOverhead;
  procedure calibrateCPU;
  procedure tablecontent(Sender: TObject; Col, Row: Longint; Rect: TRect; State: TGridDrawState);
  public
  procedure mark(nr:integer;aactive:boolean);
  end;
TZprofiler = class(TComponent)
  profiler: Tprofiler;
  constructor create(aowner:tcomponent);override;
  destructor destroy;override;
  procedure mark(nr:integer;aactive:boolean);
  end;
procedure Register;
var
  profile: Tprofiler;

implementation
{$R *.DFM}
const installedcount:integer=0;
var gridhandle:hwnd;
    mhi,mlo,nhi,nlo,overheadtime:dword;
    lastmark,CPUclock:integer;
procedure Register;
begin
  RegisterComponents('samples', [TZprofiler]);
end;

constructor TZprofiler.create(aowner:tcomponent);
begin
inherited create(aowner);
if installedcount=0 then profile:=tprofiler.create(application.mainform);
inc(installedcount);
profiler:=profile;
if (csdesigning in ComponentState) then profiler.hide
                                   else profiler.show;
end;
destructor TZprofiler.destroy;
begin
dec(installedcount);
if installedcount=0 then profile.free;
inherited destroy;
end;
procedure TZprofiler.mark(nr:integer;aactive:boolean);
begin
profiler.mark(nr,aactive);
end;

procedure tprofiler.tablecontent(Sender: TObject; Col, Row: Longint; Rect: TRect; State: TGridDrawState);
var mtext,mmtext:string;div1:extended;
begin mtext:='';
case radiogroup1.itemindex of
     0:begin mmtext:='s';div1:=1e-6;end;
     1:begin mmtext:='ms';div1:=1e-3;end;
     2:begin mmtext:='s';div1:=1;end;
     end;
if row=0 then case col of
                    0:mtext:='marknr';
                    1:mtext:='passes';
                    2:mtext:='time /'+mmtext;
                    3:mtext:='mean /'+mmtext;
                    4:mtext:='fastest /'+mmtext;
              end
          else with rmark[row] do case col of
                    0:mtext:=inttostr(row);
                    1:mtext:=inttostr(passes);
                    2:mtext:=FloatToStrF(div1*totaltime/cpuclock,fffixed,8,2);
                    3: if passes>0 then mtext:=floattostrf(div1*totaltime/(cpuclock*passes),fffixed,8,2);
                    4:if passes>0 then mtext:=FloatToStrF(div1*fastest/cpuclock,fffixed,8,2);
                    end;

with Rect, tdrawgrid(sender).Canvas do
TextRect(Rect, Left + (Right - Left - TextWidth(mtext)) div 2,
               Top + (Bottom - Top - TextHeight(mtext)) div 2, mtext);
end;

procedure mmtime;assembler;
asm
        DB 0FH
        DB 031H
        mov mhi,edx
        mov mlo,eax
end;
procedure nntime;assembler;
asm
        DB 0FH
        DB 031H
        mov nhi,edx
        mov nlo,eax
end;

procedure Tprofiler.gettime(nr:integer);
begin
       rmark[nr].thigh:=mhi;
       rmark[nr].tlow:=mlo;
end;

procedure Tprofiler.CorrectForOverhead;
var i:integer;
begin
overheadtime:=((nhi-rmark[lastmark].thigh)*$FFFFFFFF+nlo-rmark[lastmark].tlow);
for i:=1 to nmarks do with rmark[i] do
    if active then
    totaltime:=totaltime-overheadtime;
end;
function Tprofiler.gettimediff(nr:integer):extended;
begin
result:=(mhi-rmark[nr].thigh)*$FFFFFFFF+mlo-rmark[nr].tlow;
gettime(nr);
end;
procedure Tprofiler.mark(nr:integer;aactive:boolean);
var diff:extended;
begin
mmtime;
correctForOverhead;
if aactive then begin
                if(not rmark[nr].active) then with Rmark[nr] do
                begin
                gettime(nr);
                active:=true;
                end;
                end
          else begin
               if rmark[nr].active then with Rmark[nr] do
               begin
               diff:=gettimediff(nr);
               totaltime:=totaltime+diff;
               diff:=diff-overheadtime;
               active:=false;
               inc(passes);
               if passes=1 then fastest:=diff else
               if diff<fastest then fastest:=diff;
               invalidaterect(gridhandle,nil,true);
               end;
               end;
lastmark:=nr; {keep a copy for the overhead time subtraction}
nntime;
 end;

procedure Tprofiler.FormCreate(Sender: TObject);
begin
drawgrid1.ondrawcell:=tablecontent;
calibrateCPU;
drawGrid1.ColWidths[0] := 50;
drawGrid1.ColWidths[1] := 50;
gridhandle:=drawgrid1.handle;
ClearbuttonClick(Self);
end;
procedure Tprofiler.calibrateCPU;
var t1:dword;
begin
t1:=gettickcount;
while t1=gettickcount do;{wait for tickchange}
mmtime;
while gettickcount<(t1+400) do;
nntime;
CPUclock:=round(2.5e-6*((nhi-mhi)*$FFFFFFFF+nlo-mlo));{MHz}
label1.caption:='clock : '+inttostr(cpuclock)+' MHz';
end;

procedure Tprofiler.ClearbuttonClick(Sender: TObject);
var i:integer;
begin
for i:=1 to nmarks do with Rmark[i] do
                      begin
                      passes:=0;totaltime:=0;fastest:=0;
                      active:=false;
                      end;
                      invalidaterect(gridhandle,nil,true);
end;

procedure Tprofiler.RadioGroup1Click(Sender: TObject);
begin
invalidaterect(gridhandle,nil,true);
end;

procedure Tprofiler.SpeedButton1Click(Sender: TObject);
begin
calibrateCPU;
ClearbuttonClick(Self);
end;

procedure Tprofiler.SpeedButton2Click(Sender: TObject);
begin
MessageDlg('Zprofiler : freeware program timing tool by A.Baars (1997)'
, mtInformation,[mbok], 0);
end;

end.
